==> apache-paimon-0.8.1-src.tgz: paimon-0.8.1/paimon-hive/paimon-hive-connector-common/src/test/java/org/apache/paimon/hive/FlinkGenericCatalogITCase.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.hive;

import org.apache.paimon.flink.FlinkGenericCatalog;
import org.apache.paimon.flink.FlinkGenericCatalogFactory;
import org.apache.paimon.hive.annotation.Minio;
import org.apache.paimon.hive.runner.PaimonEmbeddedHiveRunner;
import org.apache.paimon.s3.MinioTestContainer;

import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableList;

import com.klarna.hiverunner.HiveShell;
import com.klarna.hiverunner.annotations.HiveSQL;
import org.apache.flink.table.api.EnvironmentSettings;
import org.apache.flink.table.api.TableEnvironment;
import org.apache.flink.table.api.internal.TableEnvironmentImpl;
import org.apache.flink.table.catalog.hive.HiveCatalog;
import org.apache.flink.table.catalog.hive.client.HiveShimLoader;
import org.apache.flink.test.util.AbstractTestBase;
import org.apache.flink.types.Row;
import org.apache.flink.util.CloseableIterator;
import org.apache.hadoop.hive.conf.HiveConf;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.junit.runner.RunWith;

import java.util.HashMap;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/** IT cases for using Flink {@code FlinkGenericCatalog}. */
@RunWith(PaimonEmbeddedHiveRunner.class)
public class FlinkGenericCatalogITCase extends AbstractTestBase {

    @Rule public TemporaryFolder folder = new TemporaryFolder();

    protected TableEnvironment tEnv;

    @HiveSQL(files = {})
    protected static HiveShell hiveShell;

    @Minio private static MinioTestContainer minioTestContainer;

    private static HiveCatalog createHiveCatalog(HiveConf hiveConf) {
        return new HiveCatalog(
                "testcatalog", null, hiveConf, HiveShimLoader.getHiveVersion(), true);
    }

    @Before
    public void before() throws Exception {
        hiveShell.execute("CREATE DATABASE IF NOT EXISTS test_db");
        hiveShell.execute("USE test_db");
        hiveShell.execute("CREATE TABLE hive_table ( a INT, b STRING )");
        hiveShell.execute("INSERT INTO hive_table VALUES (100, 'Hive'), (200, 'Table')");
        hiveShell.executeQuery("SHOW TABLES");

        tEnv = TableEnvironmentImpl.create(EnvironmentSettings.newInstance().inBatchMode().build());
        HiveCatalog hiveCatalog = createHiveCatalog(hiveShell.getHiveConf());
        FlinkGenericCatalog catalog =
                FlinkGenericCatalogFactory.createCatalog(
                        this.getClass().getClassLoader(),
                        new HashMap<>(),
                        hiveCatalog.getName(),
                        hiveCatalog);
        catalog.open();
        tEnv.registerCatalog(hiveCatalog.getName(), catalog);
        sql("USE CATALOG " + hiveCatalog.getName());
        sql("USE test_db");
    }

    @After
    public void after() {
        hiveShell.execute("DROP DATABASE IF EXISTS test_db CASCADE");
        hiveShell.execute("DROP DATABASE IF EXISTS test_db2 CASCADE");
    }

    protected List<Row> sql(String query, Object... args) {
        try (CloseableIterator<Row> iter = tEnv.executeSql(String.format(query, args)).collect()) {
            return ImmutableList.copyOf(iter);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    public void testPaimonTableToBlackHole() {
        sql(
                "CREATE TABLE paimon_t ( "
                        + "f0 INT, "
                        + "f1 INT "
                        + ") WITH ('connector'='paimon', 'file.format' = 'avro' )");
        sql("INSERT INTO paimon_t VALUES (1, 1), (2, 2)");
        assertThat(sql("SELECT * FROM paimon_t"))
                .containsExactlyInAnyOrder(Row.of(1, 1), Row.of(2, 2));

        sql("CREATE TABLE bh (f0 INT, f1 INT) WITH ('connector'='blackhole')");
        sql("INSERT INTO bh SELECT * FROM paimon_t");
    }

    @Test
    public void testReadPaimonSystemTable() {
        sql(
                "CREATE TABLE paimon_t (\n"
                        + "    user_id BIGINT,\n"
                        + "    item_id BIGINT,\n"
                        + "    behavior STRING,\n"
                        + "    dt STRING,\n"
                        + "    PRIMARY KEY (dt, user_id) NOT ENFORCED\n"
                        + ") PARTITIONED BY (dt) "
                        + " WITH ('connector'='paimon', 'file.format' = 'avro' )");
        sql("INSERT INTO paimon_t VALUES (1, 2, 'click', '2023-11-01')");
        sql("INSERT INTO paimon_t VALUES (2, 3, 'click', '2023-11-02')");

        List<Row> result =
                sql("SELECT snapshot_id, schema_id, commit_kind FROM paimon_t$snapshots");

        assertThat(result).containsExactly(Row.of(1L, 0L, "APPEND"), Row.of(2L, 0L, "APPEND"));
    }

    @Test
    public void testReadPaimonAllProcedures() {
        List<Row> result = sql("SHOW PROCEDURES");

        assertThat(result)
                .contains(Row.of("compact"), Row.of("merge_into"), Row.of("migrate_table"));
    }
}

==> apache-paimon-0.9.0-src.tgz: paimon-0.9.0/paimon-hive/paimon-hive-connector-common/src/test/java/org/apache/paimon/hive/FlinkGenericCatalogITCase.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.hive;

import org.apache.paimon.flink.FlinkGenericCatalog;
import org.apache.paimon.flink.FlinkGenericCatalogFactory;
import org.apache.paimon.hive.annotation.Minio;
import org.apache.paimon.hive.runner.PaimonEmbeddedHiveRunner;
import org.apache.paimon.s3.MinioTestContainer;

import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableList;

import com.klarna.hiverunner.HiveShell;
import com.klarna.hiverunner.annotations.HiveSQL;
import org.apache.flink.table.api.EnvironmentSettings;
import org.apache.flink.table.api.TableEnvironment;
import org.apache.flink.table.api.internal.TableEnvironmentImpl;
import org.apache.flink.table.catalog.hive.HiveCatalog;
import org.apache.flink.table.catalog.hive.client.HiveShimLoader;
import org.apache.flink.test.util.AbstractTestBaseJUnit4;
import org.apache.flink.types.Row;
import org.apache.flink.util.CloseableIterator;
import org.apache.hadoop.hive.conf.HiveConf;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.junit.runner.RunWith;

import java.util.HashMap;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/** IT cases for using Flink {@code FlinkGenericCatalog}. */
@RunWith(PaimonEmbeddedHiveRunner.class)
public class FlinkGenericCatalogITCase extends AbstractTestBaseJUnit4 {

    @Rule public TemporaryFolder folder = new TemporaryFolder();

    protected TableEnvironment tEnv;

    @HiveSQL(files = {})
    protected static HiveShell hiveShell;

    @Minio private static MinioTestContainer minioTestContainer;

    private static HiveCatalog createHiveCatalog(HiveConf hiveConf) {
        return new HiveCatalog(
                "testcatalog", null, hiveConf, HiveShimLoader.getHiveVersion(), true);
    }

    @Before
    public void before() throws Exception {
        hiveShell.execute("CREATE DATABASE IF NOT EXISTS test_db");
        hiveShell.execute("USE test_db");
        hiveShell.execute("CREATE TABLE hive_table ( a INT, b STRING )");
        hiveShell.execute("INSERT INTO hive_table VALUES (100, 'Hive'), (200, 'Table')");
        hiveShell.executeQuery("SHOW TABLES");

        tEnv = TableEnvironmentImpl.create(EnvironmentSettings.newInstance().inBatchMode().build());
        HiveCatalog hiveCatalog = createHiveCatalog(hiveShell.getHiveConf());
        FlinkGenericCatalog catalog =
                FlinkGenericCatalogFactory.createCatalog(
                        this.getClass().getClassLoader(),
                        new HashMap<>(),
                        hiveCatalog.getName(),
                        hiveCatalog);
        catalog.open();
        tEnv.registerCatalog(hiveCatalog.getName(), catalog);
        sql("USE CATALOG " + hiveCatalog.getName());
        sql("USE test_db");
    }

    @After
    public void after() {
        hiveShell.execute("DROP DATABASE IF EXISTS test_db CASCADE");
        hiveShell.execute("DROP DATABASE IF EXISTS test_db2 CASCADE");
    }

    protected List<Row> sql(String query, Object... args) {
        try (CloseableIterator<Row> iter = tEnv.executeSql(String.format(query, args)).collect()) {
            return ImmutableList.copyOf(iter);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    public void testPaimonTableToBlackHole() {
        sql(
                "CREATE TABLE paimon_t ( "
                        + "f0 INT, "
                        + "f1 INT "
                        + ") WITH ('connector'='paimon', 'file.format' = 'avro' )");
        sql("INSERT INTO paimon_t VALUES (1, 1), (2, 2)");
        assertThat(sql("SELECT * FROM paimon_t"))
                .containsExactlyInAnyOrder(Row.of(1, 1), Row.of(2, 2));

        sql("CREATE TABLE bh (f0 INT, f1 INT) WITH ('connector'='blackhole')");
        sql("INSERT INTO bh SELECT * FROM paimon_t");
    }

    @Test
    public void testReadPaimonSystemTable() {
        sql(
                "CREATE TABLE paimon_t (\n"
                        + "    user_id BIGINT,\n"
                        + "    item_id BIGINT,\n"
                        + "    behavior STRING,\n"
                        + "    dt STRING,\n"
                        + "    PRIMARY KEY (dt, user_id) NOT ENFORCED\n"
                        + ") PARTITIONED BY (dt) "
                        + " WITH ('connector'='paimon', 'file.format' = 'avro' )");
        sql("INSERT INTO paimon_t VALUES (1, 2, 'click', '2023-11-01')");
        sql("INSERT INTO paimon_t VALUES (2, 3, 'click', '2023-11-02')");

        List<Row> result =
                sql("SELECT snapshot_id, schema_id, commit_kind FROM paimon_t$snapshots");

        assertThat(result).containsExactly(Row.of(1L, 0L, "APPEND"), Row.of(2L, 0L, "APPEND"));
    }

    @Test
    public void testReadPaimonAllProcedures() {
        List<Row> result = sql("SHOW PROCEDURES");

        assertThat(result)
                .contains(Row.of("compact"), Row.of("merge_into"), Row.of("migrate_table"));
    }
}
